( 3iMusic-menu.fs - Assad Ebrahim - 2021-09-06---Mon---01:44 )

\ This is where the design will be fleshed out on what Piano V2 ought to to do


\ System turns on
\ 1 open-com
\ init-songs


: show-settings ( -- )
   cr ." songlen@ : "   songlen@  .  \ in memory  \ TODO: add songname or ID
   cr ." playmode@ : "  playmode@ . \ playmode shows watch, learn, play
   cr ." dur_ms @ : "   dur_ms @  .  \ tempo
;

\ Design: Read from mechanical selector interface, 24-bit config board
\ Design: config1 is the first byte - 1P4T and 1 1P3T made from 2x 1P2Ts.  Top position is 0001 (lsb first)
: get-config1 ( -- byte1 ) clear-rx bankB shin24 -rot 2drop ;
: check-mode ( -- 1P4Tpos ) get-config1 $0F and 4bitID ;
\ Design: Reading from 64-bit keypad, ensuring now stray rx signals queued up to corrupt the read
: showlist# ( 1-4 0-15 -- 1-4 0-15 ) 2dup swap 64 + emit 46 emit . cr ;
: full64 ( -- 0-63 ) clear-rx poll8  showlist#  1 - swap 1 - 16 * + ;

\ ===============
: scale-select ( -- ) 
    cr ." NOTE/SCALE MODE" 
    cr ." ---------------"
    cr ." Select scale ..."
   [scalelist] full64  cells  scales + @ execute ;


\ Design: this needs to be checked with each pass through the keyboard
: keyboard-select ( -- 1P3Tpos ) 
    cr ." KEYBOARD LAYOUT MODE"
    cr ." --------------------"
    cr ." Choose keyboard layout ..."
    get-config1 $F0 and 4 rshift 4bitID 1 - cells keylayout + @ execute ;

: tuner-select ( -- ) 
    cr ." TUNER MODER"
    cr ." --------------------"
    cr ." Choose stringed instrument ..."
   [tunerlist] clear-rx poll8 nip 1 - cells tuner + @ execute ;


: [song-select] ( -- ) \ song select - loads selected song to memory
    [playlist] full64  cells  songs + @  execute [writesong] ;

: song-actions (  -- ) \ song do - execute on song in memory
    songmenu-actions  cr ." Action? : ... "
    clear-rx poll8      nip  1 - cells   songsdo + @ execute ; 

: song-select ( -- ) \ song mode - loop  Ctrl+C to exit
    cr ." SONG MODE" 
    cr ." ---------"
    cr ." Select song ... "
    [song-select] begin 0 song-actions if drop exitloop else continueloop then until ;
   
\ =================

\ 1P4T - Main mode selection


\ Design: taking a simple choice from 1PT4 switch configuration - HW menu  (see 3iShift.fs)
: [choose-shp]  ( idx0-3 -- ) cells arpeg-exec + @ execute ;
\ Design: ignore error read - user will fiddle with the switch and it will work ok then
: choose-shp  ( -- ) config1P4T nip dup $FF < if [choose-shp] else cr ." Error.  Keeping previous value." then ;
: test-switch ( -- ) begin choose-shp key? until ;


: [arpeg-select]
  arpeglist full64 cells arpeg-exec + @ execute ;

: arpeg-select
    cr ." ARPEGGIO MODE" 
    cr ." -------------"
    cr ." Select chord, arpeggio, or beat ..."
    begin 0 [arpeg-select] if drop exitloop else arpeggiator continueloop then until ;


create modelist
' scale-select , ' keyboard-select , ' tuner-select , ' song-select ,  \  ' chord-select   ' arpeg-mode

\ DESIGN: notes > beats > arpeg-up > arpeg-down (real-time scan)
\   DESIGN: chord/arpeg shapes (top 4) (real-time scan) - major, minor, 7th, sus4
\ Design: notes > beats > chords/arpeg (up-down) > keyboard-wrap (high, std, low) > 
\ non-real-time modes: choose scale > instrument tuner > song player > 
: mode-select ( -- )  \ notes 
    check-mode 1 - cells  modelist + @ execute ;



\ ==============================================================
\ 2021-11-12---Fri---05:05 -- ADDING OLED SSD1306 displays to build embedded menu system
\ include 3iI2C.fs  (requires 3iI2C.avr)  include 3iFonts.fs

: piano-menu-init
oled-init
;
: piano-menu0
oled-clear
s" NANO PLAYER 328" ..oled  .cr .cr
s" Assad Ebrahim" ..oled .cr
s" Nov 2021" ..oled .cr
s" mathscitech.org" ..oled .cr
; 
: piano-menu1
oled-clear
s" 1.INSTRUMENT MODE" ..oled .cr .cr
s"   Mono" ..oled .cr
\ s"   Arpeggio" ..oled .cr
s"   Chromatic" ..oled .cr
\ s"   Wrap low" ..oled .cr
\ s"   Wrap high" ..oled .cr
s"   Equitemper" ..oled .cr
s"   Lagato" ..oled .cr
;

: piano-menu2
oled-clear
s" 2.PLAYER MODE" ..oled .cr .cr
s"   Choose Song: " ..oled .cr
.cr
.cr
.cr
s" >Jesu(Bach)" ..oled .cr
;


: piano-menu3
oled-clear
s" 3.BEATS MODE" ..oled .cr .cr
s"   Set Beat: " ..oled .cr
.cr
.cr
.cr
s" NOT ENABLED" ..oled .cr
;


: piano-menu4
oled-clear
s" 3.TUNER MODE" ..oled .cr .cr
s"   Guitar " ..oled .cr
s"   Ukelele " ..oled .cr
s"   Violin " ..oled .cr
s"   5-string Banjo " ..oled .cr
;



\ ==============================================================

: music-menu
cr ." + demos"
cr ."   ++ sound"
cr ."      - updown-slide"
cr ."   ++ lights"
cr ."      - lights-demo"
cr ."      - flashlights"
cr ."   ++ silence-2voice "
cr ."   ++ twinkle watch   twinkle-chords"
cr ."   ++ arpeg-demo "
cr ." + piano"
cr ."   ++ 13 keys, sharps/flats, continuous tone, <1 Hz tuning error, octave up/down buttons, change scale button"
cr ."    - change scale | choose scale | edit scale | transpose-up | transpose-down | ntranspose"
cr ."      ++ Western"
cr ."         cstd  (0 flats, 0 sharps)"
cr ."         fmaj  (1 flat b%, 0 sharps)"
cr ."         b%maj (2 flats b%, e%, 0 sharps)"
cr ."         e%maj (3 flats b%, e%, a%, 0 sharps)"
cr ."         a%maj (4 flats b%, e%, a%, d%, 0 sharps)"
cr ."         d%maj (5 flats b%, e%, a%, d%, g%, 0 sharps)"
cr ."         gmaj  (1 sharp c#)"
cr ."         cmaj (blues, pentatonic)"
cr ."         dmin (b%, c#)"
cr ."      ++ Non-Western (equitempered)"
cr ."           European Regional:  irish | phyrigian "
cr ."           Indian:  bhairav | purvi | todi "
cr ."           Arabian/North African: algerian | jins-hijaz "
cr ."           Far East Asian: balinese | chinese | mongolian | yu-approx "
cr ."      ++ Exotic (non-equitempered frequencies)"
cr ."           Chinese: yu | shang | gong | jue | zhi "  \ the tones of heaven
\               ADD arabic music with quarter tone frequencies
cr ." + recording"
cr ."     ++ clear-rx"
cr ."     ++ display-notes watch | replay"
cr ."     ++ get-song  watch | replay"
cr ."  + chordator"
cr ."     ++ select chord shape: "
cr ."           major  | minor  | dimin | augmen | sus4 | sus2 |"
cr ."           major7 | minor7 | halfdim7 | dimin7 | domin7"
cr ."     ++ <note > playchord (play & light)"
cr ." + beats..."
cr ." + arpeggiator"
cr ."     ++ <arpeg> n >narpeg"  \ TODO: add input arpeggio from keyboard   Ex: 1 1 2 1 3
cr ."     note#  arpeg-chord"
cr ."     arpeggiator (plays the arpeggio on base root)"
cr ." + instrument tuning"
cr ."   - guitar"
cr ."   - violin"
cr ."   - ukelele"
cr ."   - cello"
cr ."   - viola"
cr ." + songs"
cr ."   - playlist"
cr ."   - <notation> "
cr ."        1 ,,  "
cr ."        1 [,,] note" 
cr ."        <notes+rests> // /// 3/// 4/// .. "
cr ."             [...]" 
cr ."        ><<  >>>"
cr ."        1 '' (plays chords and rests)"
cr ."        c_ ;; (plays specified arpeggio on given shp@ chord, lighting chord triad)"
cr ."   - <song> play | watch | pace | learn | showuniqs | [writesong] "
cr ."   -    replay | transpose | ntranspose | [showuniqs] | "
cr ." + instrument fingering ..."
cr ." + HW menu"
cr ."    bankB sample24"
cr ."    check8 poll8"
cr ."    mode-select  scale-select  keyboard-select  tuner-select  song-select"
cr 
cr ." Execution:"
cr ." include 3iSongList.fs   1 open-com  init-songs"
cr ." help-scales  playlist  music-menu help-songlist help-pianolights  help-shiftin  help-shiftout help-sound"
;

music-menu


\ ================= VERSION HISTORY ===============================
\ 2021-10-05---Tue---22:00 - CAPABILITY: added sd [songdo] and sm [songmode] which complete user loop in song mode for song selection and action (play, tranpose, etc.)
\ 2021-10-13---Wed---05:10 - Added arpeggio selection from mechanical interface
\ 2021-11-12---Fri---05:23 - Prototyping embedded menu system on OLED 128x64 w/ SSD1306 driver IC chip

\ Design notes:
\ Sporadic changes, user initiated
\ Note mode | chord mode | tuner mode | arpeg mode
\ Note: choose scale

\ Realtime:
\ 
\  Note (scale) -> | Chord -> which type, reg or invert?| Tuner -> continuous tone, momentary, or timed pulses
